==> mastodon_to_sqlite-0.2.0-py3-none-any.whl: mastodon_to_sqlite/client.py <==
from typing import Generator, Optional, Tuple

from requests import PreparedRequest, Request, Response, Session
from requests.auth import AuthBase


class MastodonAuth(AuthBase):
    def __init__(self, access_token: str):
        self.access_token = access_token

    def __call__(self, r):
        r.headers["Authorization"] = f"Bearer {self.access_token}"
        return r


class MastodonClient:
    def __init__(self, domain: str, access_token: str):
        self.api_url = f"https://{domain}/api/v1"

        self.session = Session()
        self.session.auth = MastodonAuth(access_token)

        self.session.headers[
            "User-Agent"
        ] = "mastodon-to-sqlite (+https://github.com/myles/mastodon-to-sqlite)"

    def request(
        self,
        method: str,
        path: str,
        timeout: Optional[Tuple[int, int]] = None,
        **kwargs,
    ) -> Tuple[PreparedRequest, Response]:
        full_url = f"{self.api_url}/{path}"

        request = Request(method=method.upper(), url=full_url, **kwargs)
        prepped = self.session.prepare_request(request)
        response = self.session.send(prepped, timeout=timeout)

        return prepped, response

    def request_paginated(
        self,
        method: str,
        path: str,
        timeout: Optional[Tuple[int, int]] = None,
        **kwargs,
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        next_path: Optional[str] = path

        while next_path is not None:
            request, response = self.request(
                method=method, path=next_path, timeout=timeout, **kwargs
            )
            yield request, response

            # If there is no Link header or the Link header does not contain a
            # next link, then we know there isn't pagination this endpoint or
            # there is no next page.
            if "Link" not in response.headers or "next" not in response.links:
                next_path = None
                continue

            next_url = response.links["next"]["url"]
            next_path = next_url.replace(f"{self.api_url}/", "")

    def accounts_verify_credentials(self) -> Tuple[PreparedRequest, Response]:
        return self.request("GET", "accounts/verify_credentials")

    def accounts_followers(
        self, account_id: str
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated("GET", f"accounts/{account_id}/followers")

    def accounts_following(
        self, account_id: str
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated("GET", f"accounts/{account_id}/following")

    def accounts_statuses(
        self, account_id: str
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated("GET", f"accounts/{account_id}/statuses")

    def bookmarks(
        self,
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated("GET", "bookmarks")

    def favourites(
        self,
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated("GET", "favourites")

==> mastodon_to_sqlite-0.2.1-py3-none-any.whl: mastodon_to_sqlite/client.py <==
import datetime
from time import sleep
from typing import Generator, Optional, Tuple

from requests import PreparedRequest, Request, Response, Session
from requests.auth import AuthBase


def get_utc_now() -> datetime.datetime:
    """
    Returns the current datetime in UTC.
    """
    return datetime.datetime.now(datetime.timezone.utc)


class MastodonAuth(AuthBase):
    def __init__(self, access_token: str):
        self.access_token = access_token

    def __call__(self, r):
        r.headers["Authorization"] = f"Bearer {self.access_token}"
        return r


class MastodonClient:
    def __init__(self, domain: str, access_token: str):
        self.api_url = f"https://{domain}/api/v1"

        self.session = Session()
        self.session.auth = MastodonAuth(access_token)

        self.session.headers[
            "User-Agent"
        ] = "mastodon-to-sqlite (+https://github.com/myles/mastodon-to-sqlite)"

    def request(
        self,
        method: str,
        path: str,
        timeout: Optional[Tuple[int, int]] = None,
        **kwargs,
    ) -> Tuple[PreparedRequest, Response]:
        full_url = f"{self.api_url}/{path}"

        request = Request(method=method.upper(), url=full_url, **kwargs)
        prepped = self.session.prepare_request(request)
        response = self.session.send(prepped, timeout=timeout)

        return prepped, response

    def request_paginated(
        self,
        method: str,
        path: str,
        timeout: Optional[Tuple[int, int]] = None,
        **kwargs,
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        next_path: Optional[str] = path

        while next_path is not None:
            request, response = self.request(
                method=method, path=next_path, timeout=timeout, **kwargs
            )
            yield request, response

            # If there is no Link header or the Link header does not contain a
            # next link, then we know there isn't pagination this endpoint or
            # there is no next page.
            if "Link" not in response.headers or "next" not in response.links:
                next_path = None
                continue

            # If the Mastodon server is reporting rate limit remaining of one
            # more call, then we should sleep until we are free to call again.
            # See docs: <https://docs.joinmastodon.org/api/rate-limits/>
            if response.headers.get("X-RateLimit-Remaining") == "1":
                reset_at = datetime.datetime.fromisoformat(
                    response.headers["X-RateLimit-Reset"]
                )
                sleep((reset_at - get_utc_now()).seconds)

            next_url = response.links["next"]["url"]
            next_path = next_url.replace(f"{self.api_url}/", "")

    def accounts_verify_credentials(self) -> Tuple[PreparedRequest, Response]:
        return self.request("GET", "accounts/verify_credentials")

    def accounts_followers(
        self, account_id: str
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated(
            "GET", f"accounts/{account_id}/followers", params={"limit": "80"}
        )

    def accounts_following(
        self, account_id: str
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated(
            "GET", f"accounts/{account_id}/following", params={"limit": "80"}
        )

    def accounts_statuses(
        self, account_id: str
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated(
            "GET", f"accounts/{account_id}/statuses", params={"limit": "40"}
        )

    def bookmarks(
        self,
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated(
            "GET", "bookmarks", params={"limit": "40"}
        )

    def favourites(
        self,
    ) -> Generator[Tuple[PreparedRequest, Response], None, None]:
        return self.request_paginated(
            "GET", "favourites", params={"limit": "40"}
        )
